@@ -1,29 +1,21 @@
-#ifndef _XOPEN_SOURCE
-#define _XOPEN_SOURCE 600
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "check_compat.h"
-
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
 #include <time.h>
 
 #include <vpx/vpx_image.h>
 
+#include "../testing/misc_tools.h"
 #include "../toxav/toxav.h"
 #include "../toxcore/crypto_core.h"
 #include "../toxcore/logger.h"
 #include "../toxcore/tox.h"
 #include "../toxcore/util.h"
-
-#include "helpers.h"
+#include "check_compat.h"
 
 #define TEST_REGULAR_AV 1
 #define TEST_REGULAR_A 1
@@ -36,7 +28,7 @@
 
 
 #define ck_assert_call_control(a, b, c) do { \
-    TOXAV_ERR_CALL_CONTROL cc_err; \
+    Toxav_Err_Call_Control cc_err; \
     bool ok = toxav_call_control(a, b, c, &cc_err); \
     if (!ok) { \
         printf("toxav_call_control returned error %d\n", cc_err); \
@@ -51,6 +43,12 @@ typedef struct {
     uint32_t state;
 } CallControl;
 
+static void clear_call_control(CallControl *cc)
+{
+    const CallControl empty = {0};
+    *cc = empty;
+}
+
 
 /**
  * Callbacks
@@ -106,6 +104,55 @@ static void iterate_tox(Tox *bootstrap, Tox *Alice, Tox *Bob)
     tox_iterate(Bob, nullptr);
 }
 
+static void regular_call_flow(
+    Tox *Alice, Tox *Bob, Tox *bootstrap,
+    ToxAV *AliceAV, ToxAV *BobAV,
+    CallControl *AliceCC, CallControl *BobCC,
+    int a_br, int v_br)
+{
+    clear_call_control(AliceCC);
+    clear_call_control(BobCC);
+
+    Toxav_Err_Call call_err;
+    toxav_call(AliceAV, 0, a_br, v_br, &call_err);
+
+    if (call_err != TOXAV_ERR_CALL_OK) {
+        printf("toxav_call failed: %d\n", call_err);
+        ck_assert(0);
+    }
+
+    const time_t start_time = time(nullptr);
+
+    do {
+        if (BobCC->incoming) {
+            Toxav_Err_Answer answer_err;
+            toxav_answer(BobAV, 0, a_br, v_br, &answer_err);
+
+            if (answer_err != TOXAV_ERR_ANSWER_OK) {
+                printf("toxav_answer failed: %d\n", answer_err);
+                ck_assert(0);
+            }
+
+            BobCC->incoming = false;
+        } else { /* TODO(mannol): rtp */
+            if (time(nullptr) - start_time >= 1) {
+
+                Toxav_Err_Call_Control cc_err;
+                toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &cc_err);
+
+                if (cc_err != TOXAV_ERR_CALL_CONTROL_OK) {
+                    printf("toxav_call_control failed: %d\n", cc_err);
+                    ck_assert(0);
+                }
+            }
+        }
+
+        iterate_tox(bootstrap, Alice, Bob);
+    } while (BobCC->state != TOXAV_FRIEND_CALL_STATE_FINISHED);
+
+    printf("Success!\n");
+}
+
 static void test_av_flows(void)
 {
     Tox *Alice, *Bob, *bootstrap;
@@ -115,7 +162,7 @@ static void test_av_flows(void)
     CallControl AliceCC, BobCC;
 
     {
-        TOX_ERR_NEW error;
+        Tox_Err_New error;
 
         bootstrap = tox_new_log(nullptr, &error, &index[0]);
         ck_assert(error == TOX_ERR_NEW_OK);
@@ -148,7 +195,7 @@ static void test_av_flows(void)
 
     uint8_t off = 1;
 
-    while (1) {
+    while (true) {
         iterate_tox(bootstrap, Alice, Bob);
 
         if (tox_self_get_connection_status(bootstrap) &&
@@ -168,7 +215,7 @@ static void test_av_flows(void)
 
 
     {
-        TOXAV_ERR_NEW error;
+        Toxav_Err_New error;
         AliceAV = toxav_new(Alice, &error);
         ck_assert(error == TOXAV_ERR_NEW_OK);
 
@@ -189,80 +236,32 @@ static void test_av_flows(void)
     printf("Created 2 instances of ToxAV\n");
     printf("All set after %llu seconds!\n", time(nullptr) - cur_time);
 
-
-#define REGULAR_CALL_FLOW(A_BR, V_BR) \
-    do { \
-        memset(&AliceCC, 0, sizeof(CallControl)); \
-        memset(&BobCC, 0, sizeof(CallControl)); \
-        \
-        TOXAV_ERR_CALL call_err; \
-        toxav_call(AliceAV, 0, A_BR, V_BR, &call_err); \
-        \
-        if (call_err != TOXAV_ERR_CALL_OK) { \
-            printf("toxav_call failed: %d\n", call_err); \
-            ck_assert(0); \
-        } \
-        \
-        \
-        long long unsigned int start_time = time(nullptr); \
-        \
-        \
-        while (BobCC.state != TOXAV_FRIEND_CALL_STATE_FINISHED) { \
-            \
-            if (BobCC.incoming) { \
-                TOXAV_ERR_ANSWER answer_err; \
-                toxav_answer(BobAV, 0, A_BR, V_BR, &answer_err); \
-                \
-                if (answer_err != TOXAV_ERR_ANSWER_OK) { \
-                    printf("toxav_answer failed: %d\n", answer_err); \
-                    ck_assert(0); \
-                } \
-                BobCC.incoming = false; \
-            } else { \
-                /* TODO(mannol): rtp */ \
-                \
-                if (time(nullptr) - start_time >= 1) { \
-                    \
-                    TOXAV_ERR_CALL_CONTROL cc_err; \
-                    toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &cc_err); \
-                    \
-                    if (cc_err != TOXAV_ERR_CALL_CONTROL_OK) { \
-                        printf("toxav_call_control failed: %d\n", cc_err); \
-                        ck_assert(0); \
-                    } \
-                } \
-            } \
-             \
-            iterate_tox(bootstrap, Alice, Bob); \
-        } \
-        printf("Success!\n");\
-    } while(0)
-
     if (TEST_REGULAR_AV) {
         printf("\nTrying regular call (Audio and Video)...\n");
-        REGULAR_CALL_FLOW(48, 4000);
+        regular_call_flow(Alice, Bob, bootstrap, AliceAV, BobAV, &AliceCC, &BobCC,
+                          48, 4000);
     }
 
     if (TEST_REGULAR_A) {
         printf("\nTrying regular call (Audio only)...\n");
-        REGULAR_CALL_FLOW(48, 0);
+        regular_call_flow(Alice, Bob, bootstrap, AliceAV, BobAV, &AliceCC, &BobCC,
+                          48, 0);
     }
 
     if (TEST_REGULAR_V) {
         printf("\nTrying regular call (Video only)...\n");
-        REGULAR_CALL_FLOW(0, 4000);
+        regular_call_flow(Alice, Bob, bootstrap, AliceAV, BobAV, &AliceCC, &BobCC,
+                          0, 4000);
     }
 
-#undef REGULAR_CALL_FLOW
-
     if (TEST_REJECT) { /* Alice calls; Bob rejects */
         printf("\nTrying reject flow...\n");
 
-        memset(&AliceCC, 0, sizeof(CallControl));
-        memset(&BobCC, 0, sizeof(CallControl));
+        clear_call_control(&AliceCC);
+        clear_call_control(&BobCC);
 
         {
-            TOXAV_ERR_CALL rc;
+            Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
             if (rc != TOXAV_ERR_CALL_OK) {
@@ -271,13 +270,13 @@ static void test_av_flows(void)
             }
         }
 
-        while (!BobCC.incoming) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (!BobCC.incoming);
 
         /* Reject */
         {
-            TOXAV_ERR_CALL_CONTROL rc;
+            Toxav_Err_Call_Control rc;
             toxav_call_control(BobAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
             if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
@@ -286,9 +285,9 @@ static void test_av_flows(void)
             }
         }
 
-        while (AliceCC.state != TOXAV_FRIEND_CALL_STATE_FINISHED) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (AliceCC.state != TOXAV_FRIEND_CALL_STATE_FINISHED);
 
         printf("Success!\n");
     }
@@ -296,11 +295,11 @@ static void test_av_flows(void)
     if (TEST_CANCEL) { /* Alice calls; Alice cancels while ringing */
         printf("\nTrying cancel (while ringing) flow...\n");
 
-        memset(&AliceCC, 0, sizeof(CallControl));
-        memset(&BobCC, 0, sizeof(CallControl));
+        clear_call_control(&AliceCC);
+        clear_call_control(&BobCC);
 
         {
-            TOXAV_ERR_CALL rc;
+            Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
             if (rc != TOXAV_ERR_CALL_OK) {
@@ -309,13 +308,13 @@ static void test_av_flows(void)
             }
         }
 
-        while (!BobCC.incoming) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (!BobCC.incoming);
 
         /* Cancel */
         {
-            TOXAV_ERR_CALL_CONTROL rc;
+            Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
             if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
@@ -325,9 +324,9 @@ static void test_av_flows(void)
         }
 
         /* Alice will not receive end state */
-        while (BobCC.state != TOXAV_FRIEND_CALL_STATE_FINISHED) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (BobCC.state != TOXAV_FRIEND_CALL_STATE_FINISHED);
 
         printf("Success!\n");
     }
@@ -335,12 +334,12 @@ static void test_av_flows(void)
     if (TEST_MUTE_UNMUTE) { /* Check Mute-Unmute etc */
         printf("\nTrying mute functionality...\n");
 
-        memset(&AliceCC, 0, sizeof(CallControl));
-        memset(&BobCC, 0, sizeof(CallControl));
+        clear_call_control(&AliceCC);
+        clear_call_control(&BobCC);
 
         /* Assume sending audio and video */
         {
-            TOXAV_ERR_CALL rc;
+            Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 1000, &rc);
 
             if (rc != TOXAV_ERR_CALL_OK) {
@@ -349,9 +348,9 @@ static void test_av_flows(void)
             }
         }
 
-        while (!BobCC.incoming) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (!BobCC.incoming);
 
         /* At first try all stuff while in invalid state */
         ck_assert(!toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_PAUSE, nullptr));
@@ -362,7 +361,7 @@ static void test_av_flows(void)
         ck_assert(!toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_SHOW_VIDEO, nullptr));
 
         {
-            TOXAV_ERR_ANSWER rc;
+            Toxav_Err_Answer rc;
             toxav_answer(BobAV, 0, 48, 4000, &rc);
 
             if (rc != TOXAV_ERR_ANSWER_OK) {
@@ -407,7 +406,7 @@ static void test_av_flows(void)
         ck_assert(BobCC.state & TOXAV_FRIEND_CALL_STATE_ACCEPTING_V);
 
         {
-            TOXAV_ERR_CALL_CONTROL rc;
+            Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
             if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
@@ -425,12 +424,12 @@ static void test_av_flows(void)
     if (TEST_STOP_RESUME_PAYLOAD) { /* Stop and resume audio/video payload */
         printf("\nTrying stop/resume functionality...\n");
 
-        memset(&AliceCC, 0, sizeof(CallControl));
-        memset(&BobCC, 0, sizeof(CallControl));
+        clear_call_control(&AliceCC);
+        clear_call_control(&BobCC);
 
         /* Assume sending audio and video */
         {
-            TOXAV_ERR_CALL rc;
+            Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
             if (rc != TOXAV_ERR_CALL_OK) {
@@ -439,12 +438,12 @@ static void test_av_flows(void)
             }
         }
 
-        while (!BobCC.incoming) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (!BobCC.incoming);
 
         {
-            TOXAV_ERR_ANSWER rc;
+            Toxav_Err_Answer rc;
             toxav_answer(BobAV, 0, 48, 0, &rc);
 
             if (rc != TOXAV_ERR_ANSWER_OK) {
@@ -475,7 +474,7 @@ static void test_av_flows(void)
         ck_assert(!(BobCC.state & TOXAV_FRIEND_CALL_STATE_SENDING_A));
 
         {
-            TOXAV_ERR_CALL_CONTROL rc;
+            Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
             if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
@@ -493,12 +492,12 @@ static void test_av_flows(void)
     if (TEST_PAUSE_RESUME_SEND) { /* Stop and resume audio/video payload and test send options */
         printf("\nTrying stop/resume functionality...\n");
 
-        memset(&AliceCC, 0, sizeof(CallControl));
-        memset(&BobCC, 0, sizeof(CallControl));
+        clear_call_control(&AliceCC);
+        clear_call_control(&BobCC);
 
         /* Assume sending audio and video */
         {
-            TOXAV_ERR_CALL rc;
+            Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
             if (rc != TOXAV_ERR_CALL_OK) {
@@ -507,12 +506,12 @@ static void test_av_flows(void)
             }
         }
 
-        while (!BobCC.incoming) {
+        do {
             iterate_tox(bootstrap, Alice, Bob);
-        }
+        } while (!BobCC.incoming);
 
         {
-            TOXAV_ERR_ANSWER rc;
+            Toxav_Err_Answer rc;
             toxav_answer(BobAV, 0, 48, 0, &rc);
 
             if (rc != TOXAV_ERR_ANSWER_OK) {
@@ -535,7 +534,7 @@ static void test_av_flows(void)
         iterate_tox(bootstrap, Alice, Bob);
 
         {
-            TOXAV_ERR_CALL_CONTROL rc;
+            Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
             if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
