@@ -33,7 +33,7 @@ if [ "${SUPPORT_ARCH_x86_64}" = "true" ]; then
         gcc-mingw-w64-x86-64
 fi
 
-# Pacakges needed for running toxcore tests
+# Packages needed for running toxcore tests
 if [ "${SUPPORT_TEST}" = "true" ]; then
     apt-get install -y \
         apt-transport-https \
@@ -43,7 +43,12 @@ if [ "${SUPPORT_TEST}" = "true" ]; then
 
     # Add Wine package repository to use the latest Wine
     echo "deb https://dl.winehq.org/wine-builds/debian/ stretch main" >> /etc/apt/sources.list
-    curl -o Release.key https://dl.winehq.org/wine-builds/Release.key
+    curl -o Release.key https://dl.winehq.org/wine-builds/winehq.key
+    # Verify against a known good key fingerprint. --dry-run makes it so we don't actually import the key.
+    if ! gpg --batch --dry-run --import --import-options import-show --fingerprint Release.key | grep 'D43F 6401 4536 9C51 D786  DDEA 76F1 A20F F987 672F'; then
+        echo "Error: WineHQ's Debian package repository key fingerprint didn't match the expected one. Exiting."
+        exit 1
+    fi
     apt-key add Release.key
 
     dpkg --add-architecture i386
