@@ -1,5 +1,5 @@
 /*
- * Copyright © 2016-2017 The TokTok team.
+ * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2014 Tox project.
  *
  * This file is part of Tox, the free peer to peer instant messenger.
@@ -17,29 +17,38 @@
  * You should have received a copy of the GNU General Public License
  * along with Tox.  If not, see <http://www.gnu.org/licenses/>.
  */
+#ifndef C_TOXCORE_TOXAV_GROUPAV_H
+#define C_TOXCORE_TOXAV_GROUPAV_H
+
 #include "../toxcore/group.h"
+#include "../toxcore/tox.h"
 
 /* Audio encoding/decoding */
 #include <opus.h>
 
 #define GROUP_AUDIO_PACKET_ID 192
 
+// TODO(iphydf): Use this better typed one instead of the void-pointer one below.
+// typedef void audio_data_cb(Tox *tox, uint32_t groupnumber, uint32_t peernumber, const int16_t *pcm,
+//                            uint32_t samples, uint8_t channels, uint32_t sample_rate, void *userdata);
+typedef void audio_data_cb(void *tox, uint32_t groupnumber, uint32_t peernumber, const int16_t *pcm,
+                           uint32_t samples, uint8_t channels, uint32_t sample_rate, void *userdata);
+
 /* Create a new toxav group.
  *
  * return group number on success.
  * return -1 on failure.
  */
-int add_av_groupchat(Logger *log, Group_Chats *g_c, void (*audio_callback)(Messenger *, uint32_t, uint32_t,
-                     const int16_t *, unsigned int, uint8_t, uint32_t, void *), void *userdata);
+int add_av_groupchat(const Logger *log, Tox *tox, Group_Chats *g_c, audio_data_cb *audio_callback, void *userdata);
 
 /* Join a AV group (you need to have been invited first.)
  *
  * returns group number on success
  * returns -1 on failure.
  */
-int join_av_groupchat(Logger *log, Group_Chats *g_c, uint32_t friendnumber, const uint8_t *data, uint16_t length,
-                      void (*audio_callback)(Messenger *, uint32_t, uint32_t, const int16_t *, unsigned int, uint8_t, uint32_t, void *),
-                      void *userdata);
+int join_av_groupchat(const Logger *log, Tox *tox, Group_Chats *g_c, uint32_t friendnumber, const uint8_t *data,
+                      uint16_t length,
+                      audio_data_cb *audio_callback, void *userdata);
 
 
 /* Send audio to the group chat.
@@ -50,3 +59,5 @@ int join_av_groupchat(Logger *log, Group_Chats *g_c, uint32_t friendnumber, cons
 int group_send_audio(Group_Chats *g_c, uint32_t groupnumber, const int16_t *pcm, unsigned int samples, uint8_t channels,
                      uint32_t sample_rate);
 
+
+#endif // C_TOXCORE_TOXAV_GROUPAV_H
