@@ -1,5 +1,5 @@
 /*
- * Copyright © 2016-2017 The TokTok team.
+ * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013-2015 Tox project.
  *
  * This file is part of Tox, the free peer to peer instant messenger.
@@ -35,13 +35,11 @@
  *
  * Note that total size of pcm in bytes is equal to (samples * channels * sizeof(int16_t)).
  */
-int toxav_add_av_groupchat(Tox *tox, void (*audio_callback)(void *, uint32_t, uint32_t, const int16_t *, unsigned int,
-                           uint8_t, uint32_t, void *), void *userdata)
+int toxav_add_av_groupchat(Tox *tox, audio_data_cb *audio_callback, void *userdata)
 {
-    Messenger *m = (Messenger *)tox;
-    return add_av_groupchat(m->log, (Group_Chats *)m->conferences_object,
-                            (void (*)(Messenger *, uint32_t, uint32_t, const int16_t *, unsigned int, uint8_t, uint32_t, void *))audio_callback,
-                            userdata);
+    // TODO(iphydf): Don't rely on toxcore internals.
+    Messenger *m = *(Messenger **)tox;
+    return add_av_groupchat(m->log, tox, m->conferences_object, audio_callback, userdata);
 }
 
 /* Join a AV group (you need to have been invited first.)
@@ -55,14 +53,11 @@ int toxav_add_av_groupchat(Tox *tox, void (*audio_callback)(void *, uint32_t, ui
  * Note that total size of pcm in bytes is equal to (samples * channels * sizeof(int16_t)).
  */
 int toxav_join_av_groupchat(Tox *tox, uint32_t friendnumber, const uint8_t *data, uint16_t length,
-                            void (*audio_callback)
-                            (void *, uint32_t, uint32_t, const int16_t *, unsigned int, uint8_t, uint32_t, void *),
-                            void *userdata)
+                            audio_data_cb *audio_callback, void *userdata)
 {
-    Messenger *m = (Messenger *)tox;
-    return join_av_groupchat(m->log, (Group_Chats *)m->conferences_object, friendnumber, data, length,
-                             (void (*)(Messenger *, uint32_t, uint32_t, const int16_t *, unsigned int, uint8_t, uint32_t, void *))audio_callback,
-                             userdata);
+    // TODO(iphydf): Don't rely on toxcore internals.
+    Messenger *m = *(Messenger **)tox;
+    return join_av_groupchat(m->log, tox, m->conferences_object, friendnumber, data, length, audio_callback, userdata);
 }
 
 /* Send audio to the group chat.
@@ -81,6 +76,7 @@ int toxav_join_av_groupchat(Tox *tox, uint32_t friendnumber, const uint8_t *data
 int toxav_group_send_audio(Tox *tox, uint32_t groupnumber, const int16_t *pcm, unsigned int samples, uint8_t channels,
                            uint32_t sample_rate)
 {
-    Messenger *m = (Messenger *)tox;
-    return group_send_audio((Group_Chats *)m->conferences_object, groupnumber, pcm, samples, channels, sample_rate);
+    // TODO(iphydf): Don't rely on toxcore internals.
+    Messenger *m = *(Messenger **)tox;
+    return group_send_audio(m->conferences_object, groupnumber, pcm, samples, channels, sample_rate);
 }
